<?php

/*
  ----------------------------------------------------------------------
  LICENSE

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License (GPL)
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  To read the license please visit http://www.gnu.org/copyleft/gpl.html
  ----------------------------------------------------------------------
*/

/**
 * Install the MonSyndicat WordPress distribution in the default domain
 * of new accounts.
 *
 * @copyright 2018 Kienan Stewart <kienan@koumbit.org>
 */
class m_monsyndicat_wp {

  function hook_admin_add_member() {
    global $msg, $cuid, $L_FQDN, $L_HOSTING, $mysql, $mem;
    // Use $cuid to look up new member name
    // Use $mysql->add_db($member_name . '_default' to create the database
    //
    $db = new DB_System();
    if (!$db->query("SELECT m.*, parent.login as parentlogin FROM membres m LEFT JOIN membres parent ON parent.uid=m.creator WHERE m.uid= ?", array($cuid))) {
       $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("query failed: %s "), $db->Error));
       return false;
    }
    $db_name = '';
    if ($db->next_record()) {
      $db_name = $db->Record['login'] . '_default';
      $r = $mysql->add_db($db_name);
      if (!$r) {
        $msg->raise("ERROR", "admin", sprintf(_("Failed to create new database: %s"), $name . '_default'));
	return false;
      }
    } else {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("Query failed: %s"), $db->Error));
      return false;
    }

    // DB Creation succeeded.
    $msg->log('monsyndicat_wp', 'hook_admin_add_member', 'Created default database');
    return true;
    // 1. Get the default domain and path, db login, pass, name
    $domain = NULL;
    $domain_path = NULL;
    $db_name = NULL;
    $db_user = NULL;
    $db_pass = NULL;
    $admin_mail = NULL;
    $query = "select m.login, m.uid, m.mail, s.domaine, s.sub, s.valeur, s.type, db.login as 'db_login', db.pass, db.db from membres m left join sub_domaines s on m.uid = s.compte left join db on m.uid = db.uid where s.type = 'vhost' and m.uid = ?";
    if (!$db->query($query, array($cuid))) {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(domain/dbinfo) Query failed: %s"), $db->Error));
      return false;
    }
    if ($db->next_record()) {
      $domain =  $db->Record['sub'] . '.' . $db->Record['domaine'];
      $domain_path = $db->Record['valeur'];
      $db_name = $db->Record['db'];
      $db_user = $db->Record['db_login'];
      $db_pass = $db->Record['pass'];
      $admin_mail = $db->Record['mail'];
    }
    else {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(domain/dbinfo) Query has no results: %s"), $db->Error));
      return false;
      $query = "select m.login, m.uid, m.mail, db.login as 'db_login', db.pass, db.pass, db.db from membres m left join db on m.uid = db.uid where m.uid = ?";
      if(!$db->query($query, array($cuid))) {
        $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(alternate dbinfo) Query failed: %s"), $db->Error));
        return false;
      }
      if ($db->next_record()) {
        $domain = $db->Record['login'];
      }
    }
    // 2. Get the database host, port (seems to hardcoded to 3306)
    // @TODO support more than one database
    $db_host = NULL;
    $db_port = 3306;
    $query = "select * from db_servers where client ='%'";
    if (!$db->query($query)) {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(dbhost) Query failed: %s"), $db->Error));
      return false;
    }
    if ($db->next_record()) {
      $db_host = $db->Record['host'];
    } else {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(dbhost) Query has no results: %s"), $db->Error));
      return false;
    }
    // 3. Unpack WP and run configuration steps
    // @TODO make this more generic / parameterizable
    // @TODO allow parametrization of WP executable in AlternC install
    // @TODO fewer su_shell_exec calls
    // @TODO better place and parametrizable archive path
    $output = '';
    $output .= $this->su_shell_exec("tar -x -C $domain_path -f /tmp/wp-dist-csn-latest.tar", $cuid);
    $output .= $this->su_shell_exec("rm $domain_path/wp-config.php", $cuid);
    $output .= $this->su_shell_exec("cd $domain_path ; wp config create --dbname=\"$db_name\" --dbuser=\"$db_user\" --dbhost=\"$db_host\" --dbpass=\"$db_pass\" --locale='fr_CA'", $cuid);
    $output .= $this->su_shell_exec("cd $domain_path ; wp core install --url=\"$domain\" --title=\"@TODO\" --admin_email=\"$admin_mail\" --admin_user='monsyndicat_wp'", $cuid);
    // @TODO Detect errors.
    $msg->log("monsyndicat_wp", "hook_admin_add_member", "WP deployment: " . $output);
    return true;
  }

  public function hook_dom_add_domain($id) {
      global $msg, $cuid, $L_FQDN, $L_HOSTING, $mysql, $mem, $L_ALTERNC_HTML, $L_ALTERNC_MONSYNDICAT_TAR, $action;
    if (!isset($L_ALTERNC_MONSYNDICAT_TAR)) {
       $msg->raise("ERROR". 'monsyndicat_wp', 'ALTERNC_MONSYNDICAT_TAR must be set in /etc/alternc/locals.sh');
       return false;
    }
    // Get owner uid
    $db = new DB_System();
    $sub_id = NULL;
    $msg->log('monsyndicat_wp', 'hook_dom_add_domain', 'Domain ID: ' . $id);
    $q = 'select sd.sub, sd.valeur, sd.domaine, sd.compte from sub_domaines sd left join domaines d on sd.domaine = d.domaine where sd.type = "vhost" and d.id = ?';
    if (!$db->query($q, array($id))) {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(domaininfo) Query failed: %s"), $db->Error));
      return false;
    }

    $domain = NULL;
    $domain_path = NULL;
    $owner = NULL;
    if ($db->next_record()) {
      $domain = $db->Record['sub'] . '.' . $db->Record['domaine'];
      $domain_path = $db->Record['valeur'];
      $owner = $db->Record['compte'];
    } else {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(domaininfo) Query has no results"), $db->Error));
      return false;
    }

    $db_name = NULL;
    $db_user = NULL;
    $db_pass = NULL;
    $admin_mail = NULL;
    $login = NULL;
    $query = "select m.login, m.mail, db.login as 'db_login', db.pass, db.db from membres m left join db on m.uid = db.uid where m.uid = ?";
    if (!$db->query($query, array($owner))) {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(user/dbinfo) Query failed: %s"), $db->Error));
      return false;
    }
    if ($db->next_record()) {
      $db_name = $db->Record['db'];
      $db_user = $db->Record['db_login'];
      $db_pass = $db->Record['pass'];
      $admin_mail = $db->Record['mail'];
      $login = $db->Record['login'];
    }
    else {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(user/dbinfo) Query has no results: %s"), $db->Error));
      return false;
    }

    // 2. Get the database host, port (seems to hardcoded to 3306)
    // @TODO support more than one database
    $db_host = NULL;
    $db_port = 3306;
    $query = "select * from db_servers where client ='%'";
    if (!$db->query($query)) {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(dbhost) Query failed: %s"), $db->Error));
      return false;
    }
    if ($db->next_record()) {
      $db_host = $db->Record['host'];
    } else {
      $msg->raise("ERROR", "monsyndicat_wp", sprintf(_("(dbhost) Query has no results: %s"), $db->Error));
      return false;
    }

    // 3. Unpack WP and run configuration steps
    // @TODO make this more generic / parameterizable
    // @TODO allow parametrization of WP executable in AlternC install
    // @TODO fewer su_shell_exec calls
    // @TODO better place and parametrizable archive path
    //$mem->su($owner);
    #$output = '';
    #$output .= $this->su_shell_exec("tar -x -C $domain_path -f /root/wp-dist-csn-latest.tar", $cuid);
    #$output .= $this->su_shell_exec("rm $domain_path/wp-config.php", $cuid);
    #$output .= $this->su_shell_exec("cd $domain_path ; wp config create --dbname=\"$db_name\" --dbuser=\"$db_user\" --dbhost=\"$db_host\" --dbpass=\"$db_pass\" --locale='fr_CA'", $cuid);
    #$output .= $this->su_shell_exec("cd $domain_path ; wp core install --url=\"$domain\" --title=\"@TODO\" --admin_email=\"$admin_mail\" --admin_user='monsyndicat_wp'", $cuid);
    $output = '';
    $domain_path = $L_ALTERNC_HTML . '/' . substr($login, 0, 1) . '/' . $login . '/' . $domain_path;
    $output .= shell_exec("tar -x -C $domain_path -f $L_ALTERNC_MONSYNDICAT_TAR 2>&1");
    $output .= shell_exec("rm -f $domain_path/wp-config.php 2>&1");
    $output .= shell_exec("cd $domain_path ; wp config create --dbname=\"$db_name\" --dbuser=\"$db_user\" --dbhost=\"$db_host\" --dbpass=\"$db_pass\" --locale='fr_CA' 2>&1");
    $output .= shell_exec("cd $domain_path ; wp core install --url=\"$domain\" --title=\"@TODO\" --admin_email=\"$admin_mail\" --admin_user='monsyndicat_wp' 2>&1");
    // @TODO Detect errors.
    //$mem->unsu();
    // Queue fix-permissions
    $action->fix_user($owner);
    $msg->log("monsyndicat_wp", "hook_dom_add_domain", "WP deployment: " . $output);
    return true;
  }

  /*
   * Wrapper for shell_exec().
   *
   * @returns string
   *   The output of shell_exec.
   */
  private function su_shell_exec($cmd, $uid, $shell = '/bin/bash') {
    global $msg;
    if (is_numeric($uid)) {
      $uid = '#' . $uid;
    }
    $r = shell_exec("sudo -u '$uid' -s '$shell' \"$cmd\"");
    if ($r === NULL) {
      $msg->log('monsyndicat_wp', 'su_shell_exec', 'Command (as uid ' . $uid . ') might have error: ' . $cmd);
    }
  }
}
